@@ -16,7 +16,7 @@ from ducktape.tests.test import Test
 from ducktape.mark import matrix
 from ducktape.mark import parametrize
 
-NUM_TESTS = 15
+NUM_TESTS = 17
 
 
 class TestMatrix(Test):
@@ -44,3 +44,11 @@ class TestParametrized(Test):
     def test_thing(self, x, y):
         """2 tests"""
         pass
+
+
+class TestObjectParameters(Test):
+    @parametrize(d={'a': 'A'}, lst=['whatever'])
+    @parametrize(d={'z': 'Z'}, lst=['something'])
+    def test_thing(self, d, lst):
+        """2 tests"""
+        pass
